"""
Spatelier - Personal tool library for video and music file handling.
"""

from __future__ import annotations

__version__ = "0.4.2"
__author__ = "Galen Spikes"
__email__ = "galenspikes@gmail.com"
